@@ -106,7 +106,7 @@ void decode_primary_volume(const uint8_t *p, struct primary_volume_descriptor *p
     memcpy(pvd->volume_set_identifier, p + 72, 128);
 }
 
-/* Anchor Volume Description Pointer (ECMA 167 3/10.2) */
+/* Anchor Volume Descriptor Pointer (ECMA 167 3/10.2) */
 void decode_avdp(const uint8_t *p, struct anchor_volume_descriptor *avdp)
 {
     /* Main volume descriptor sequence extent */
@@ -116,6 +116,12 @@ void decode_avdp(const uint8_t *p, struct anchor_volume_descriptor *avdp)
     _decode_extent_ad(p + 24, &avdp->rvds);
 }
 
+/* Volume Descriptor Pointer (ECMA 167 3/10.3) */
+void decode_vdp(const uint8_t *p, struct volume_descriptor_pointer *vdp)
+{
+    _decode_extent_ad(p + 20, &vdp->next_extent);
+}
+
 /* Partition Descriptor (ECMA 167 3/10.5) */
 void decode_partition(const uint8_t *p, struct partition_descriptor *pd)
 {
@@ -144,6 +150,11 @@ void decode_logical_volume(const uint8_t *p, struct logical_volume_descriptor *l
         map_size = sizeof(lvd->partition_map_table);
     }
 
+    /* input size is one block (2048 bytes) */
+    if (map_size > 2048 - 440) {
+        map_size = 2048 - 440;
+    }
+
     memcpy(lvd->partition_map_table, p + 440, map_size);
 }
 
@@ -158,16 +169,26 @@ void decode_file_set_descriptor(const uint8_t *p, struct file_set_descriptor *fs
 }
 
 /* File Identifier (ECMA 167 4/14.4) */
-size_t decode_file_identifier(const uint8_t *p, struct file_identifier *fi)
+size_t decode_file_identifier(const uint8_t *p, size_t size, struct file_identifier *fi)
 {
     size_t l_iu; /* length of implementation use field */
 
+    if (size < 38) {
+        ecma_error("not enough data\n");
+        return 0;
+    }
+
     fi->characteristic = _get_u8(p + 18);
     fi->filename_len   = _get_u8(p + 19);
     decode_long_ad(p + 20, &fi->icb);
 
     l_iu = _get_u16(p + 36);
 
+    if (size < 38 + l_iu + fi->filename_len) {
+        ecma_error("not enough data\n");
+        return 0;
+    }
+
     if (fi->filename_len) {
         memcpy(fi->filename, p + 38 + l_iu, fi->filename_len);
     }
@@ -300,16 +321,17 @@ static struct file_entry *_decode_file_entry(const uint8_t *p, size_t size,
 
     fe->file_type = tag.file_type;
     fe->length    = _get_u64(p + 56);
-    fe->num_ad    = num_ad;
     fe->ad_type   = tag.flags & 7;
 
     if (content_inline) {
         /* data of small files can be embedded in file entry */
         /* copy embedded file data */
         fe->content_inline = 1;
-        memcpy(fe->data.content, p + p_ad, l_ad);
+        fe->u.data.information_length = l_ad;
+        memcpy(fe->u.data.content, p + p_ad, l_ad);
     } else {
-        _decode_file_ads(p + p_ad, fe->ad_type, partition, &fe->data.ad[0], num_ad);
+        fe->u.ads.num_ad = num_ad;
+        _decode_file_ads(p + p_ad, fe->ad_type, partition, &fe->u.ads.ad[0], num_ad);
     }
 
     return fe;
@@ -339,21 +361,15 @@ int decode_allocation_extent(struct file_entry **p_fe, const uint8_t *p, size_t
         return 0;
     }
 
-    fe = (struct file_entry *)realloc(fe, sizeof(struct file_entry) + sizeof(struct long_ad) * (fe->num_ad + num_ad - 1));
+    fe = (struct file_entry *)realloc(*p_fe, sizeof(struct file_entry) + sizeof(struct long_ad) * (fe->u.ads.num_ad + num_ad));
     if (!fe) {
         return -1;
     }
     *p_fe = fe;
 
-    /* drop pointer to this extent from the end of ads */
-    if (fe->data.ad[fe->num_ad - 1].extent_type != ECMA_AD_EXTENT_AD) {
-        ecma_error("decode_allocation_extent: missing link ad\n");
-    }
-    fe->num_ad--;
-
     /* decode new allocation descriptors */
-    _decode_file_ads(p + 24, fe->ad_type, partition, &fe->data.ad[fe->num_ad], num_ad);
-    fe->num_ad += num_ad;
+    _decode_file_ads(p + 24, fe->ad_type, partition, &fe->u.ads.ad[fe->u.ads.num_ad], num_ad);
+    fe->u.ads.num_ad += num_ad;
 
     return 0;
 }
