@@ -343,19 +343,23 @@ static void *_jvm_dlopen_a(const char *java_home,
 static void *_load_jli_macos()
 {
     const char *java_home = NULL;
-    static const char jli_dir[] = "jre/lib/jli";
+    static const char * const jli_dir[]  = {
+        "jre/lib/jli", "lib/jli",
+    };
+    const unsigned num_jli_dir  = sizeof(jli_dir)  / sizeof(jli_dir[0]);
+
     static const char jli_lib[] = "libjli";
     void *handle;
 
     /* JAVA_HOME set, use it */
     java_home = getenv("JAVA_HOME");
     if (java_home) {
-        return _jvm_dlopen(java_home, jli_dir, jli_lib);
+        return _jvm_dlopen_a(java_home, jli_dir, num_jli_dir, jli_lib);
     }
 
     java_home = _java_home_macos();
     if (java_home) {
-        handle = _jvm_dlopen(java_home, jli_dir, jli_lib);
+        handle = _jvm_dlopen_a(java_home, jli_dir, num_jli_dir, jli_lib);
         if (handle) {
             return handle;
         }
@@ -486,7 +490,7 @@ static int _can_read_file(const char *fn)
     return 0;
 }
 
-void bdj_storage_cleanup(BDJ_STORAGE *p)
+void bdj_config_cleanup(BDJ_CONFIG *p)
 {
     X_FREE(p->cache_root);
     X_FREE(p->persistent_root);
@@ -587,7 +591,7 @@ static char *_find_libbluray_jar1(const char *jar0)
     return jar1;
 }
 
-static int _find_libbluray_jar(BDJ_STORAGE *storage)
+static int _find_libbluray_jar(BDJ_CONFIG *storage)
 {
     if (!storage->classpath[0]) {
         storage->classpath[0] = _find_libbluray_jar0();
@@ -607,7 +611,7 @@ static int _find_libbluray_jar(BDJ_STORAGE *storage)
     return !!storage->classpath[0];
 }
 
-static const char *_bdj_persistent_root(BDJ_STORAGE *storage)
+static const char *_bdj_persistent_root(BDJ_CONFIG *storage)
 {
     const char *root;
     char       *data_home;
@@ -638,7 +642,7 @@ static const char *_bdj_persistent_root(BDJ_STORAGE *storage)
     return storage->persistent_root;
 }
 
-static const char *_bdj_buda_root(BDJ_STORAGE *storage)
+static const char *_bdj_buda_root(BDJ_CONFIG *storage)
 {
     const char *root;
     char       *cache_home;
@@ -696,7 +700,7 @@ static int _get_method(JNIEnv *env, jclass *cls, jmethodID *method_id,
 }
 
 static int _bdj_init(JNIEnv *env, struct bluray *bd, const char *disc_root, const char *bdj_disc_id,
-                     BDJ_STORAGE *storage)
+                     BDJ_CONFIG *storage)
 {
     if (!bdj_register_native_methods(env)) {
         BD_DEBUG(DBG_BDJ | DBG_CRIT, "Couldn't register native methods.\n");
@@ -738,7 +742,7 @@ static int _bdj_init(JNIEnv *env, struct bluray *bd, const char *disc_root, cons
     return 1;
 }
 
-int bdj_jvm_available(BDJ_STORAGE *storage)
+int bdj_jvm_available(BDJ_CONFIG *storage)
 {
     const char *java_home;
     void* jvm_lib = _load_jvm(&java_home);
@@ -964,11 +968,11 @@ static int _create_jvm(void *jvm_lib, const char *java_home, const char *jar_fil
 }
 
 BDJAVA* bdj_open(const char *path, struct bluray *bd,
-                 const char *bdj_disc_id, BDJ_STORAGE *storage)
+                 const char *bdj_disc_id, BDJ_CONFIG *cfg)
 {
     BD_DEBUG(DBG_BDJ, "bdj_open()\n");
 
-    if (!_find_libbluray_jar(storage)) {
+    if (!_find_libbluray_jar(cfg)) {
         BD_DEBUG(DBG_BDJ | DBG_CRIT, "BD-J start failed: " BDJ_JARFILE " not found.\n");
         return NULL;
     }
@@ -1000,7 +1004,7 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
 
     JNIEnv* env = NULL;
     JavaVM *jvm = NULL;
-    const char *jar[2] = { storage->classpath[0], storage->classpath[1] };
+    const char *jar[2] = { cfg->classpath[0], cfg->classpath[1] };
     if (!_find_jvm(jvm_lib, &env, &jvm) &&
         !_create_jvm(jvm_lib, java_home, jar, &env, &jvm)) {
 
@@ -1020,7 +1024,7 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
         BD_DEBUG(DBG_BDJ, "Java version: %d.%d\n", version >> 16, version & 0xffff);
     }
 
-    if (!_bdj_init(env, bd, path, bdj_disc_id, storage)) {
+    if (!_bdj_init(env, bd, path, bdj_disc_id, cfg)) {
         bdj_close(bdjava);
         return NULL;
     }
