@@ -20,8 +20,7 @@
 package javax.media;
 
 public class StopAtTimeEvent extends StopEvent {
-    public StopAtTimeEvent(Controller source, int previous, int current,
-            int target, Time mediaTime) {
+    public StopAtTimeEvent(Controller source, int previous, int current, int target, Time mediaTime) {
         super(source, previous, current, target, mediaTime);
     }
 
